@@ -32,13 +32,14 @@
 #' reload all Hannanum analyzer dictionary 
 #'
 #' Mainly, user dictionary reloading for Hannanum Analyzer. 
-#' If you want to update user dictionary on Sejong_pkg_dir/inst/dics/data/kE/dic_user.txt, need to execute this function after editing dic.
+#' If you want to update user dictionary on KoNLP_dic/current/dic_user.txt, need to execute this function after editing dictionary.
 #'
 #' @examples
 #' \dontrun{
 #' ## This codes can not be run if you don't have encoding system which can en/decode Hangul(ex) CP949, EUC-KR, UTF-8).  
-#' dicpath <- paste(system.file(package="Sejong"), "/dics/data/kE/dic_user2.txt", sep="")
-#' newdic <- read.table(dicpath, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
+#' dicpath <- paste(system.file(package="Sejong"), "/dics/handic.zip", sep="")
+#' conn <- unz(dicpath, "data/kE/dic_user2.txt")
+#' newdic <- read.table(conn, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
 #' mergeUserDic(newdic)
 #' ## backup merged new dictionary
 #' backupUsrDic(ask=FALSE)
@@ -51,10 +52,39 @@ reloadAllDic <- function(){
   if(!exists("HannanumObj", envir=KoNLP:::.KoNLPEnv)){
     assign("HannanumObj",.jnew("HannanumInterface"),KoNLP:::.KoNLPEnv)
   }
-  .jcall(get("HannanumObj",envir=KoNLP:::.KoNLPEnv), , "reloadAllDic")
+  .jcall(get("HannanumObj",envir=KoNLP:::.KoNLPEnv), "V", "reloadAllDic")
 }
 
 
+
+# reload dictionaries for specific functions
+#
+# This function for reloading user dictionary for specific functions,
+# after you have updated user dictionary on KoNLP_dic/current/user_dic.txt.
+# 
+# @param whichDics character vector which can be "extractNoun", "SimplePos09", "SimplePos22", "SimplePos22"
+# @examples 
+# \dontrun{
+# reloadUserDic(c("extractNoun", "SimplePos22"))} 
+# @export
+reloadUserDic <- function(whichDics){
+  if(!exists("HannanumObj", envir=KoNLP:::.KoNLPEnv)){
+    assign("HannanumObj",.jnew("HannanumInterface"),KoNLP:::.KoNLPEnv)
+  }
+  if(!is.character(whichDics)){
+    stop("'whichDics' must be character!")
+  }
+  for(dic in whichDics){
+    ret <- .jcall(get("HannanumObj",envir=KoNLP:::.KoNLPEnv), 
+                  "I", "reloadUserDic", get("CurrentUserDic", envir=KoNLP:::.KoNLPEnv), dic)
+    if(ret < 0){
+      cat(sprintf("Dictionaries in %s was not reloaded\n", dic))
+    }
+  }
+}
+
+
+
 #' tag name converter
 #' 
 #' only suppport tag convertion between KAIST and Sejong tag set.
@@ -72,8 +102,43 @@ convertTag <-function(fromTag, toTag, tag){
 }
 
 
+#' use Sejong noun dictionary
+#'
+#' Retrive Sejong dictionary to use in KoNLP
+#' 
+#' @param backup will backup current dictionary?
+#' @export
+useSejongDic <- function(backup=T){
+  useDic("Sejong", backup)
+}
 
 
+# internal function to change dictionary
+useDic <- function(dicname, backup=T){
+  if(dicname == "Sejong"){
+    relpath <- "data/kE/dic_user2.txt"
+  }else if(dicname == "System"){
+    relpath <- "data/kE/dic_user.txt" 
+  }else{
+    stop("wrong dictionary name!")
+  }
+  newdic <- readZipDic(get("SejongDicsZip", envir=KoNLP:::.KoNLPEnv), relpath)
+  if(backup == T){
+    backupUsrDic(ask=F)
+  }
+  mergeUserDic(newdic,append=F)
+}
+
+#' use system default dictionary
+#'
+#' Retrive system default dictionary to use in KoNLP
+#' 
+#' @param backup will backup current dictionary?
+#' @export
+useSystemDic <- function(backup=T){
+  useDic("System", backup)
+}
+
 #' use for backup current dic_user.txt
 #'  
 #' Utility function for backup dic_user.txt file to backup directory.
@@ -81,8 +146,9 @@ convertTag <-function(fromTag, toTag, tag){
 #' @examples
 #' \dontrun{
 #' ## This codes can not be run if you don't have encoding system which can en/decode Hangul(ex) CP949, EUC-KR, UTF-8). 
-#' dicpath <- paste(system.file(package="Sejong"), "/dics/data/kE/dic_user2.txt", sep="")
-#' newdic <- read.table(dicpath, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
+#' dicpath <- paste(system.file(package="Sejong"), "/dics/handic.zip", sep="")
+#' conn <- unz(dicpath, "data/kE/dic_user2.txt")
+#' newdic <- read.table(conn, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
 #' mergeUserDic(newdic)
 #' ## backup merged new dictionary
 #' backupUsrDic(ask=FALSE)
@@ -93,23 +159,22 @@ convertTag <-function(fromTag, toTag, tag){
 #' @param ask ask to confirm backup
 #' @export
 backupUsrDic <- function(ask=TRUE){
-  UserDicPath <- get("UserDic",envir=KoNLP:::.KoNLPEnv)
-  alteredUserDicPath <- get("backupUserDic", KoNLP:::.KoNLPEnv)
+  UserDic <- get("CurrentUserDic",envir=KoNLP:::.KoNLPEnv)
+  alteredUserDicPath <- get("backupUserDicPath", KoNLP:::.KoNLPEnv)
   response <- "Y"
   if(ask){
     response <- readline("Would you backup your current 'dic_user.txt' file to backup directory? (Y/n): ")
   }
   if(substr(response,1,1) == "Y"){
-    dicpath <- get("backupUserDicPath",KoNLP:::.KoNLPEnv)
     ret1 <- TRUE
-    if(!file.exists(dicpath)){
-      ret1 <- dir.create(dicpath)
+    if(!file.exists(alteredUserDicPath)){
+      ret1 <- dir.create(alteredUserDicPath)
     }
-    ret2 <- file.copy(UserDicPath, alteredUserDicPath,overwrite=T)
+    ret2 <- file.copy(UserDic, alteredUserDicPath,overwrite=T)
     if(ret1 && ret2){
       cat("Backup was just finished!\n")  
     }else{
-      warning(sprintf("Could not copy %s\n", UserDicPath))
+      warning(sprintf("Could not copy %s\n", UserDic))
       assign("CopyedUserDic", FALSE, KoNLP:::.KoNLPEnv)
     }
   }
@@ -124,8 +189,9 @@ backupUsrDic <- function(ask=TRUE){
 #' @examples
 #' \dontrun{
 #' ## This codes can not be run if you don't have encoding system which can en/decode Hangul(ex) CP949, EUC-KR, UTF-8). 
-#' dicpath <- paste(system.file(package="Sejong"), "/dics/data/kE/dic_user2.txt", sep="")
-#' newdic <- read.table(dicpath, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
+#' dicpath <- paste(system.file(package="Sejong"), "/dics/handic.zip", sep="")
+#' conn <- unz(dicpath, "data/kE/dic_user2.txt")
+#' newdic <- read.table(conn, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
 #' mergeUserDic(newdic)
 #' ## backup merged new dictionary
 #' backupUsrDic(ask=FALSE)
@@ -142,7 +208,7 @@ restoreUsrDic <- function(ask=TRUE){
   if(!file.exists(get("backupUserDic", KoNLP:::.KoNLPEnv))){
     stop("There is no backuped dic_user.txt to restore!\n")
   }
-  UserDicPath <- get("UserDic",envir=KoNLP:::.KoNLPEnv)
+  UserDicPath <- get("CurrentUserDicPath",envir=KoNLP:::.KoNLPEnv)
   alteredUserDicPath <- get("backupUserDic", KoNLP:::.KoNLPEnv)
   response <- "Y"
   if(ask){
@@ -156,6 +222,7 @@ restoreUsrDic <- function(ask=TRUE){
       warning(sprintf("Could not copy %s\n", UserDicPath))
     }
   }
+  reloadAllDic()
 }
 
 
@@ -166,8 +233,9 @@ restoreUsrDic <- function(ask=TRUE){
 #' @examples
 #' \dontrun{
 #' ## This codes can not be run if you don't have encoding system which can en/decode Hangul(ex) CP949, EUC-KR, UTF-8). 
-#' dicpath <- paste(system.file(package="Sejong"), "/dics/data/kE/dic_user2.txt", sep="")
-#' newdic <- read.table(dicpath, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
+#' dicpath <- paste(system.file(package="Sejong"), "/dics/handic.zip", sep="")
+#' conn <- unz(dicpath, "data/kE/dic_user2.txt")
+#' newdic <- read.table(conn, sep="\t", header=FALSE, fileEncoding="UTF-8", stringsAsFactors=FALSE)
 #' mergeUserDic(newdic)
 #' ## backup merged new dictionary
 #' backupUsrDic(ask=FALSE)
@@ -195,8 +263,8 @@ mergeUserDic <- function(newUserDic, append=TRUE, verbose=FALSE){
     stop("Unsupported tag names!\n")
   }
   #combine with current dic_user.txt or replace them all.   
-  UserDicPath <- get("UserDic",envir=KoNLP:::.KoNLPEnv)
-  oldUserDic <- read.table(UserDicPath, sep="\t", header=F, fileEncoding="UTF-8", stringsAsFactors=F)
+  UserDic <- get("CurrentUserDic",envir=KoNLP:::.KoNLPEnv)
+  oldUserDic <- read.table(UserDic, sep="\t", header=F, fileEncoding="UTF-8", stringsAsFactors=F)
 
   newDicEnc <- unique(Encoding(newUserDic[,1]))
   if(length(newDicEnc) > 1){
@@ -216,13 +284,20 @@ mergeUserDic <- function(newUserDic, append=TRUE, verbose=FALSE){
   }else{
     newestUserDic <- newUserDic
   }
-  write.table(newestUserDic,file=UserDicPath,quote=F,row.names=F, sep="\t", col.names=F,fileEncoding="UTF-8")  
+  write.table(newestUserDic,file=UserDic,quote=F,row.names=F, sep="\t", col.names=F,fileEncoding="UTF-8")  
   cat(sprintf("%s words were added to dic_user.txt.\n", nrow(newUserDic)))
+  reloadAllDic()
 }
 
 
 
 
+readZipDic <- function(zipPath, dicPath){
+  dicvector <- .jcall("kr/pe/freesearch/KoNLP/KoNLPUtil", 
+                      "[S", "readZipDic", zipPath, dicPath)
+  Encoding(dicvector) <- "UTF-8"
+  return(as.data.frame(matrix(dicvector,ncol=2,byrow=T), stringsAsFactors=F))
+}
 
 
 
